// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum (gir tilgang til en rekke fields (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * Subclass of the abstract class Material, 
 * which supplies all necessary common methods for its subclasses.<br>
 * BrassMaterial supplies methods that are particular to the component.<br>
 * Used for making a brass material for the different stands of the hourglass.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class BrassMaterial extends Material
{
    /**
     * BrassMaterial constructor
     */
    public BrassMaterial( GLFunc gl)
    {
        super(gl);
    }
    
    /**
     * Enables the BrassMaterial
     * @param sideChoice the constant to indicate which side 
     * of the objekt should have this material, can be GL_FRONT_AND_BACK, 
     * GL_FRONT or GL_BACK. 
     */
    public void enableMaterial(int sideChoice)
    {
        // Hvis det er materiale p begge sider skal det ogs reagere p lys p begge sider
        if (sideChoice == GL_FRONT_AND_BACK)
        {
            gl.glLightModeli(GL_LIGHT_MODEL_TWO_SIDE,1);
        }
        // Messing
        float amb[]={0.399412f, 0.223529f, 0.027451f, 1.0f};
        float diff[]={0.780392f, 0.568627f, 0.113725f, 1.0f};
        float spec[]={0.992157f, 0.941176f, 0.807843f, 1.0f};
        float shine=27.8974f;
        
        gl.glMaterialfv(sideChoice, GL_AMBIENT,amb);
        gl.glMaterialfv(sideChoice, GL_DIFFUSE,diff);
        gl.glMaterialfv(sideChoice, GL_SPECULAR,spec);
        gl.glMaterialf(sideChoice, GL_SHININESS,shine);
    }
}