// Importerer ndvendige pakker
import gl4java.*;			// For  kunne implementere GLEnum (gir tilgang til en rekke fields (variablene i store bokstaver))
import gl4java.utils.glut.*;	// Open GL's "verktyboks"

/**
 * <strong>Description:</strong><br>
 * Classic onion-shaped bulbs.<br>
 * <br><strong>All code is extensively commented in Norwegian.</strong>
 * @author Therese Rsholdt, Grafisk Databehandling 2003, Hi
 * @version 0.1
 */
public class ClassicBulbs extends Bulbs
{
	// INSTANSIERER VARIABLER SOM HAR SPESIELL VERDI FOR DENNE KLASSEN
	// Glassets type
	String cBulbsTypeName = "Classic";
	// Lengden p halve glasset, fra midten og nedover/oppover (hele glasset 6.0f)
	float cBulbheight = 3.0f;
	// Plassering p Y-aksen
	float cCtrz0=0.0f*cBulbheight; 	// Y = 0
	float cCtrz1=0.1f*cBulbheight; 	// Y = 0.3
	float cCtrz2=0.3f*cBulbheight; 	// Y = 0.9
	float cCtrz3=0.5f*cBulbheight;	// Y = 1.5
	float cCtrz4=cBulbheight;	    	// Y = 3
	// Lengden p radiene til glasset, fra midten av glasset og nedover/oppover
	// cBRad0 og cBRad4 blir lik radien til timeglasset p det aktuelle stedet
	// cBRad1, cBRad2 og cBRad3 er radier til kontrollpunkter, ikke til selve glasset
	// Plassering p X-aksen
	float cBRad0=0.02f*cBulbheight; //startpunkt (midt p selve timeglasset)
	float cBRad1=0.02f*cBulbheight; 
	float cBRad2=0.4f*cBulbheight;
	float cBRad3=0.7f*cBulbheight;
	float cBRad4=0.5f*cBulbheight; //sluttpunkt (bunnen/toppen av glasset)
	
	// Matrisearray med alle kontrollpunktene til glasset
	float cBulbCtrPoints[]= {
			// u=0 , startpunkt, Y = 0    
			-cBRad0,  cCtrz0,  0.0f,	//v=0
			-cBRad0,  cCtrz0,  rf1*cBRad0,	//v=1
			0.0f,  cCtrz0,  rf*cBRad0,	//v=2
			cBRad0,  cCtrz0,  rf1*cBRad0,	//v=3
			cBRad0,  cCtrz0,  0.0f 	//v=4
			,
			// u=1
			-cBRad1,  cCtrz1,  0.0f,	//v=0
			-cBRad1,  cCtrz1,  rf1*cBRad1,	//v=1
			0.0f,  cCtrz1,  rf*cBRad1,	//v=2
			cBRad1,  cCtrz1,  rf1*cBRad1,	//v=3
			cBRad1,  cCtrz1,  0.0f	//v=4
			,
			// u=2
			-cBRad2,  cCtrz2,  0.0f,	//v=0
			-cBRad2,  cCtrz2,  rf1*cBRad2,	//v=1
			0.0f,  cCtrz2,  rf*cBRad2,	//v=2
			cBRad2,  cCtrz2,  rf1*cBRad2,	//v=3
			cBRad2,  cCtrz2,  0.0f	//v=4
			,
			// u=3
			-cBRad3,  cCtrz3,  0.0f,	//v=0
			-cBRad3,  cCtrz3,  rf1*cBRad3,	//v=1
			0.0f,  cCtrz3,  rf*cBRad3,	//v=2
			cBRad3,  cCtrz3,  rf1*cBRad3,	//v=3
			cBRad3,  cCtrz3,  0.0f 	//v=4
			,
			// u=4, sluttpunkt, Y = cBulbheight
			-cBRad4,  cCtrz4,  0.0f,		//v=0
			-cBRad4,  cCtrz4,  rf1*cBRad4,		//v=1
			0.0f,  cCtrz4,  rf*cBRad4,	//v=2
			cBRad4,  cCtrz4,  rf1*cBRad4,		//v=3
			cBRad4,  cCtrz4,  0.0f }; 	//v=4
							
	// "Tykkelsen" p glasset
	float cGlassThickness = 0.05f;
	// Lengden p radiene til sanden, fra midten av timeglasset og nedover/oppover
	// Plassering p X-aksen
	float cSRad0 = cBRad0 - cGlassThickness; //startpunkt
	float cSRad1 = cBRad1 - cGlassThickness;
	float cSRad2 = cBRad2 - cGlassThickness;
	float cSRad3 = cBRad3 - cGlassThickness;
	float cSRad4 = cBRad4 - cGlassThickness; //sluttpunkt
	
	// Matrisearray med alle kontrollpunktene til sanden
	float cSandCtrPoints [] = {
		// u=0 , startpunkt, z = 0    
		-cSRad0,  cCtrz0,  0.0f,	//v=0
		-cSRad0,  cCtrz0,  rf1*cSRad0,	//v=1
		0.0f,  cCtrz0,  rf*cSRad0,	//v=2
		cSRad0,  cCtrz0,  rf1*cSRad0,	//v=3
		cSRad0,  cCtrz0,  0.0f	//v=4
		,
		// u=1
		-cSRad1,  cCtrz1,  0.0f,	//v=0
		-cSRad1,  cCtrz1,  rf1*cSRad1,	//v=1
		0.0f,  cCtrz1,  rf*cSRad1,	//v=2
		cSRad1,  cCtrz1,  rf1*cSRad1,	//v=3
		cSRad1,  cCtrz1,  0.0f 	//v=4
		,
		// u=2
		-cSRad2,  cCtrz2,  0.0f,	//v=0
		-cSRad2,  cCtrz2,  rf1*cSRad2,	//v=1
		0.0f,  cCtrz2,  rf*cSRad2,	//v=2
		cSRad2,  cCtrz2,  rf1*cSRad2,	//v=3
		cSRad2,  cCtrz2,  0.0f	//v=4
		,
		// u=3
		-cSRad3,  cCtrz3,  0.0f,	//v=0
		-cSRad3,  cCtrz3,  rf1*cSRad3,	//v=1
		0.0f,  cCtrz3,  rf*cSRad3,	//v=2
		cSRad3,  cCtrz3,  rf1*cSRad3,	//v=3
		cSRad3,  cCtrz3,  0.0f 	//v=4
		,
		//2u=4, sluttpunkt, z = cBulbheight
		-cSRad4,  cCtrz4,  0.0f,		//v=0
		-cSRad4,  cCtrz4,  rf1*cSRad4,		//v=1
		0.0f,  cCtrz4,  rf*cSRad4,	//v=2
		cSRad4,  cCtrz4,  rf1*cSRad4, 		//v=3
		cSRad4,  cCtrz4,  0.0f }; 	//v=4
	
	// Arrayer som inneholder kontrollpunktene til sandens Bezierkurve (opp langs glasset)
	// Brukes til  regne ut X-Y-Z-verdier for et punkt p sandens kurve
	float cSandBezierCurvePoints0[] = {cSRad0, cCtrz0, 0.0f}; // startpunkt
	float cSandBezierCurvePoints1[] = {cSRad1, cCtrz1, 0.0f};
	float cSandBezierCurvePoints2[] = {cSRad2, cCtrz2, 0.0f};
	float cSandBezierCurvePoints3[] = {cSRad3, cCtrz3, 0.0f};
	float cSandBezierCurvePoints4[] = {cSRad4, cCtrz4, 0.0f}; // endepunkt
	
	// VARIABLER FOR SANDEN I DEN NEDERSTE DELEN AV GLASSET
	// Det hyeste sandnivet
	float cMaxLowerSandLevel = 0.1f;
	// Utganspunkt for  sette radien til den lile haugen (lowerTinyHeap)
	float cLowerTinyHeapRadiusStart = 0.1f; 
	// Hvor mange steg som behves fr lowerTinyHeap nr glassets vegger
	int cLowerTinyHeapSteps = 45;
	// Hvor mange steg som behves for Beziersanden  n maxLowerSandLevel
	int cBottomLowerSandSteps = 83;
	
	// VARIABLER FOR SANDEN I DEN VERSTE DELEN AV GLASSET
	// Det hyeste sandnivet
	float cMaxUpperSandLevel = 0.8f;
	// Disse 4 variablene bestemmer med hvor mye sanden skal synke
	float cUpperSandShrink1 = 0.0015f;
	float cUpperSandShrink2 = 0.0033f;
	float cUpperSandShrink3 = 0.006f;
	float cUpperSandShrink4 = 0.01f;
	// Disse 4 variablene regulerer hvor mange steg som skal tas
	// med  en gitt upperSandShrink-verdi
	// cLowerTinyHeapSteps + cBottomLowerSandSteps = cUpperSandSteps1 + cUpperSandSteps2 + cUpperSandSteps3 + cUpperSandSteps4
	int cUpperSandSteps1 = 45;
	int cUpperSandSteps2 = 40;
	int cUpperSandSteps3 = 35;
	int cUpperSandSteps4 = 8;
	
	/**
	* ClassicBulbs constructor
	* @param gl the <code>GLFunc</code> interface that is used for this component
	* @param glu the <code>GLUFunc</code> interface that is used for this component
	* @param glut the <code>GLUTFunc</code> interface that is used for this component
	*/
	public ClassicBulbs( GLFunc gl, GLUFunc glu, GLUTFunc glut)
	{
		super(gl, glu, glut);
		setSpecialBulbValues(); 
	}
	
	/**
	* Sets all the variables particular to the subclass.<br>
	*/
	public void setSpecialBulbValues()
	{
		setMaxLowerSandLevel(cMaxLowerSandLevel);
		setBottomLowerSandSteps(cBottomLowerSandSteps);
		setLowerTinyHeapRadiusStart(cLowerTinyHeapRadiusStart);
		setLowerTinyHeapSteps (cLowerTinyHeapSteps);
		
		setMaxUpperSandLevel(cMaxUpperSandLevel);
		setUpperSandShrinks(cUpperSandShrink1, cUpperSandShrink2, cUpperSandShrink3, cUpperSandShrink4);
		setUpperSandSteps(cUpperSandSteps1, cUpperSandSteps2, cUpperSandSteps3, cUpperSandSteps4);
		
		setBulbsTypeName(cBulbsTypeName);
		setBulbheight(	cBulbheight);
		setAllControlPoints(cCtrz0, cCtrz1, cCtrz2, cCtrz3, cCtrz4, 
							cBRad0, cBRad1, cBRad2, cBRad3, cBRad4,
							cSRad0, cSRad1, cSRad2, cSRad3, cSRad4);
		setGlassThickness(cGlassThickness);
		setBulbCtrPoints(cBulbCtrPoints);
		setSandCtrPoints(cSandCtrPoints);
		setSandBezierCurvePoints0(cSandBezierCurvePoints0); // startpunkt
		setSandBezierCurvePoints1(cSandBezierCurvePoints1);
		setSandBezierCurvePoints2(cSandBezierCurvePoints2);
		setSandBezierCurvePoints3(cSandBezierCurvePoints3);
		setSandBezierCurvePoints4(cSandBezierCurvePoints4); // endepunkt
	}
}